@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -60,6 +58,7 @@ static guint signals[LAST_SIGNAL];
 enum {
   PROP_0,
   PROP_MODE,
+  PROP_USERNAME,
   PROP_LAST,
 };
 
@@ -73,6 +72,7 @@ struct _GisDriverPrivate {
   const gchar *user_password;
 
   gchar *lang_id;
+  gchar *username;
 
   GisDriverMode mode;
 };
@@ -87,6 +87,7 @@ gis_driver_finalize (GObject *object)
   GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
 
   g_free (priv->lang_id);
+  g_free (priv->username);
 
   G_OBJECT_CLASS (gis_driver_parent_class)->finalize (object);
 }
@@ -97,12 +98,12 @@ prepare_main_window (GisDriver *driver)
   GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
   GdkGeometry size_hints;
 
-  size_hints.min_width = 680;
-  size_hints.min_height = 440;
+  size_hints.min_width = 1024;
+  size_hints.min_height = 768;
   size_hints.win_gravity = GDK_GRAVITY_CENTER;
 
   gtk_window_set_geometry_hints (priv->main_window,
-                                 GTK_WIDGET (priv->main_window),
+                                 NULL,
                                  &size_hints,
                                  GDK_HINT_MIN_SIZE | GDK_HINT_WIN_GRAVITY);
 
@@ -110,11 +111,10 @@ prepare_main_window (GisDriver *driver)
                            gis_assistant_get_titlebar (priv->assistant));
 }
 
-static gboolean
+static void
 rebuild_pages (GisDriver *driver)
 {
   g_signal_emit (G_OBJECT (driver), signals[REBUILD_PAGES], 0);
-  return FALSE;
 }
 
 GisAssistant *
@@ -139,6 +139,21 @@ gis_driver_get_user_language (GisDriver *driver)
   return priv->lang_id;
 }
 
+void
+gis_driver_set_username (GisDriver *driver, const gchar *username)
+{
+  GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
+  g_free (priv->username);
+  priv->username = g_strdup (username);
+  g_object_notify (G_OBJECT (driver), "username");
+}
+
+const gchar *
+gis_driver_get_username (GisDriver *driver)
+{
+  GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
+  return priv->username;
+}
 void
 gis_driver_set_user_permissions (GisDriver   *driver,
                                  ActUser     *user,
@@ -171,7 +186,7 @@ static void
 gis_driver_real_locale_changed (GisDriver *driver)
 {
   GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
-  g_idle_add ((GSourceFunc) rebuild_pages, driver);
+  rebuild_pages (driver);
   gis_assistant_locale_changed (priv->assistant);
 }
 
@@ -201,6 +216,9 @@ gis_driver_get_property (GObject      *object,
     case PROP_MODE:
       g_value_set_enum (value, priv->mode);
       break;
+    case PROP_USERNAME:
+      g_value_set_string (value, priv->username);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -220,6 +238,10 @@ gis_driver_set_property (GObject      *object,
     case PROP_MODE:
       priv->mode = g_value_get_enum (value);
       break;
+    case PROP_USERNAME:
+      g_free (priv->username);
+      priv->username = g_value_dup_string (value);
+      break;
    default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -259,7 +281,6 @@ gis_driver_startup (GApplication *app)
   priv->main_window = g_object_new (GTK_TYPE_APPLICATION_WINDOW,
                                     "application", app,
                                     "type", GTK_WINDOW_TOPLEVEL,
-                                    "border-width", 12,
                                     "icon-name", "preferences-system",
                                     "resizable", FALSE,
                                     "window-position", GTK_WIN_POS_CENTER_ALWAYS,
@@ -322,6 +343,11 @@ gis_driver_class_init (GisDriverClass *klass)
                        GIS_DRIVER_MODE_EXISTING_USER,
                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
+  obj_props[PROP_USERNAME] =
+    g_param_spec_string ("username", "", "",
+                         NULL,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class, PROP_LAST, obj_props);
 }
 
