@@ -195,6 +195,8 @@ main (int argc, char *argv[])
     { NULL }
   };
 
+  g_unsetenv ("GIO_USE_VFS");
+
   context = g_option_context_new (_("- GNOME initial setup"));
   g_option_context_add_main_entries (context, entries, NULL);
 
@@ -217,6 +219,8 @@ main (int argc, char *argv[])
   }
 #endif
 
+  gis_ensure_login_keyring ("gis");
+
   driver = gis_driver_new (get_mode ());
   g_signal_connect (driver, "rebuild-pages", G_CALLBACK (rebuild_pages_cb), NULL);
   status = g_application_run (G_APPLICATION (driver), argc, argv);
