@@ -31,6 +31,7 @@
 #include <string.h>
 #include <act/act-user-manager.h>
 #include "um-utils.h"
+#include "um-photo-dialog.h"
 
 #define GOA_API_IS_SUBJECT_TO_CHANGE
 #include <goa/goa.h>
@@ -41,14 +42,21 @@
 
 struct _GisAccountPageLocalPrivate
 {
+  GtkWidget *avatar_button;
   GtkWidget *avatar_image;
   GtkWidget *subtitle;
   GtkWidget *fullname_entry;
   GtkWidget *username_combo;
+  UmPhotoDialog *photo_dialog;
+
+  GdkPixbuf *avatar_pixbuf;
+  gchar *avatar_filename;
 
   ActUser *act_user;
   ActUserManager *act_client;
 
+  GoaClient *goa_client;
+
   gboolean valid_name;
   gboolean valid_username;
   ActUserAccountType account_type;
@@ -71,7 +79,7 @@ validation_changed (GisAccountPageLocal *page)
   g_signal_emit (page, signals[VALIDATION_CHANGED], 0);
 }
 
-static void
+static gboolean
 get_profile_sync (const gchar        *access_token,
                   gchar             **out_name,
                   gchar             **out_picture,
@@ -83,9 +91,9 @@ get_profile_sync (const gchar        *access_token,
   RestProxyCall *call;
   JsonParser *parser;
   JsonObject *json_object;
-  gchar *ret;
+  gboolean ret;
 
-  ret = NULL;
+  ret = FALSE;
 
   identity_error = NULL;
   proxy = NULL;
@@ -101,6 +109,7 @@ get_profile_sync (const gchar        *access_token,
 
   if (!rest_proxy_call_sync (call, error))
     goto out;
+
   if (rest_proxy_call_get_status_code (call) != 200)
     {
       g_set_error (error,
@@ -129,6 +138,8 @@ get_profile_sync (const gchar        *access_token,
       goto out;
     }
 
+  ret = TRUE;
+
   json_object = json_node_get_object (json_parser_get_root (parser));
   if (out_name != NULL)
     *out_name = g_strdup (json_object_get_string_member (json_object, "name"));
@@ -142,77 +153,88 @@ get_profile_sync (const gchar        *access_token,
     g_object_unref (call);
   if (proxy != NULL)
     g_object_unref (proxy);
+
+  return ret;
 }
 
 static void
 prepopulate_account_page (GisAccountPageLocal *page)
 {
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
-  GoaClient *client;
-  GError *error = NULL;
   gchar *name = NULL;
   gchar *picture = NULL;
   GdkPixbuf *pixbuf = NULL;
 
-  priv->valid_name = FALSE;
-  priv->valid_username = FALSE;
-
-  /* FIXME: change this for a large deployment scenario; maybe through a GSetting? */
-  priv->account_type = ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR;
-
-  client = goa_client_new_sync (NULL, &error);
-  if (client) {
+  if (priv->goa_client) {
     GList *accounts, *l;
-    accounts = goa_client_get_accounts (client);
+    accounts = goa_client_get_accounts (priv->goa_client);
     for (l = accounts; l != NULL; l = l->next) {
       GoaOAuth2Based *oa2;
       oa2 = goa_object_get_oauth2_based (GOA_OBJECT (l->data));
       if (oa2) {
-        gchar *token;
-        goa_oauth2_based_call_get_access_token_sync (oa2, &token, NULL, NULL, NULL);
-        get_profile_sync (token, &name, &picture, NULL, NULL);
+        gchar *token = NULL;
+        GError *error = NULL;
+        if (!goa_oauth2_based_call_get_access_token_sync (oa2, &token, NULL, NULL, &error))
+          {
+            g_warning ("Couldn't get oauth2 token: %s\n", error->message);
+            g_error_free (error);
+          }
+        else if (!get_profile_sync (token, &name, &picture, NULL, &error))
+          {
+            g_warning ("Couldn't get profile information: %s\n", error->message);
+            g_error_free (error);
+          }
         /* FIXME: collect information from more than one account
          * and present at least the pictures in the avatar chooser
          */
+        g_print ("Got profile information: Name '%s' Picture '%s'\n", name, picture);
         break;
       }
     }
     g_list_free_full (accounts, (GDestroyNotify) g_object_unref);
-    g_object_unref (client);
   }
 
   if (name) {
     gtk_label_set_text (GTK_LABEL (priv->subtitle), _("Are these the right details? You can change them if you want."));
     gtk_entry_set_text (GTK_ENTRY (priv->fullname_entry), name);
   }
-  else {
-    gtk_label_set_text (GTK_LABEL (priv->subtitle), _("We need a few details to complete setup."));
-    gtk_entry_set_text (GTK_ENTRY (priv->fullname_entry), "");
-    gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (priv->username_combo))));
-  }
 
   if (picture) {
     GFile *file;
-    GInputStream *stream;
+    GFileInputStream *stream;
+    GError *error = NULL;
     file = g_file_new_for_uri (picture);
-    stream = G_INPUT_STREAM (g_file_read (file, NULL, NULL));
-    pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream, -1, 96, TRUE, NULL, NULL);
-    g_object_unref (stream);
+    stream = g_file_read (file, NULL, &error);
+    if (!stream)
+      {
+        g_warning ("Failed to read picture: %s\n", error->message);
+        g_error_free (error);
+      }
+    else
+      {
+        pixbuf = gdk_pixbuf_new_from_stream_at_scale (G_INPUT_STREAM (stream), -1, 96, TRUE, NULL, NULL);
+        g_object_unref (stream);
+      }
     g_object_unref (file);
   }
 
   if (pixbuf) {
     gtk_image_set_from_pixbuf (GTK_IMAGE (priv->avatar_image), pixbuf);
-  }
-  else {
-    gtk_image_set_pixel_size (GTK_IMAGE (priv->avatar_image), 96);
-    gtk_image_set_from_icon_name (GTK_IMAGE (priv->avatar_image), "avatar-default-symbolic", 1);
+    priv->avatar_pixbuf = pixbuf;
   }
 
   g_free (name);
   g_free (picture);
 }
 
+static void
+accounts_changed (GoaClient *client, GoaObject *object, gpointer data)
+{
+  GisAccountPageLocal *page = data;
+
+  prepopulate_account_page (page);
+}
+
 static void
 fullname_changed (GtkWidget      *w,
                   GParamSpec     *pspec,
@@ -270,6 +292,37 @@ username_changed (GtkComboBoxText     *combo,
   validation_changed (page);
 }
 
+static void
+avatar_callback (GdkPixbuf   *pixbuf,
+                 const gchar *filename,
+                 gpointer     user_data)
+{
+  GisAccountPageLocal *page = user_data;
+  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
+  GdkPixbuf *tmp;
+
+  g_clear_object (&priv->avatar_pixbuf);
+  g_free (priv->avatar_filename);
+  priv->avatar_filename = NULL;
+
+  if (pixbuf) {
+    priv->avatar_pixbuf = g_object_ref (pixbuf);
+    tmp = gdk_pixbuf_scale_simple (pixbuf, 96, 96, GDK_INTERP_BILINEAR);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (priv->avatar_image), tmp);
+    g_object_unref (tmp);
+  }
+  else if (filename) {
+    priv->avatar_filename = g_strdup (filename);
+    tmp = gdk_pixbuf_new_from_file_at_size (filename, 96, 96, NULL);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (priv->avatar_image), tmp);
+    g_object_unref (tmp);
+  }
+  else {
+    gtk_image_set_pixel_size (GTK_IMAGE (priv->avatar_image), 96);
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->avatar_image), "avatar-default-symbolic", 1);
+  }
+}
+
 static void
 gis_account_page_local_constructed (GObject *object)
 {
@@ -285,7 +338,82 @@ gis_account_page_local_constructed (GObject *object)
   g_signal_connect (priv->username_combo, "changed",
                     G_CALLBACK (username_changed), page);
 
-  prepopulate_account_page (page);
+  priv->valid_name = FALSE;
+  priv->valid_username = FALSE;
+
+  /* FIXME: change this for a large deployment scenario; maybe through a GSetting? */
+  priv->account_type = ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR;
+
+  gtk_label_set_text (GTK_LABEL (priv->subtitle), _("We need a few details to complete setup."));
+  gtk_entry_set_text (GTK_ENTRY (priv->fullname_entry), "");
+  gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (priv->username_combo))));
+
+  gtk_image_set_pixel_size (GTK_IMAGE (priv->avatar_image), 96);
+  gtk_image_set_from_icon_name (GTK_IMAGE (priv->avatar_image), "avatar-default-symbolic", 1);
+
+  priv->goa_client = goa_client_new_sync (NULL, NULL);
+  if (priv->goa_client) {
+    g_signal_connect (priv->goa_client, "account-added",
+                      G_CALLBACK (accounts_changed), page);
+    g_signal_connect (priv->goa_client, "account-removed",
+                      G_CALLBACK (accounts_changed), page);
+    prepopulate_account_page (page);
+  }
+
+  priv->photo_dialog = um_photo_dialog_new (priv->avatar_button,
+                                            avatar_callback,
+                                            page);
+}
+
+static void
+gis_account_page_local_dispose (GObject *object)
+{
+  GisAccountPageLocal *page = GIS_ACCOUNT_PAGE_LOCAL (object);
+  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
+
+  g_clear_object (&priv->goa_client);
+  g_clear_object (&priv->avatar_pixbuf);
+  g_clear_pointer (&priv->avatar_filename, g_free);
+  g_clear_pointer (&priv->photo_dialog, um_photo_dialog_free);
+
+  G_OBJECT_CLASS (gis_account_page_local_parent_class)->dispose (object);
+}
+
+static void
+set_user_avatar (GisAccountPageLocal *page)
+{
+  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
+  GFile *file = NULL;
+  GFileIOStream *io_stream = NULL;
+  GOutputStream *stream = NULL;
+  GError *error = NULL;
+
+  if (priv->avatar_filename != NULL) {
+    act_user_set_icon_file (priv->act_user, priv->avatar_filename);
+    return;
+  }
+
+  if (priv->avatar_pixbuf == NULL) {
+    return;
+  }
+
+  file = g_file_new_tmp ("usericonXXXXXX", &io_stream, &error);
+  if (error != NULL)
+    goto out;
+
+  stream = g_io_stream_get_output_stream (G_IO_STREAM (io_stream));
+  if (!gdk_pixbuf_save_to_stream (priv->avatar_pixbuf, stream, "png", NULL, &error, NULL))
+    goto out;
+
+  act_user_set_icon_file (priv->act_user, g_file_get_path (file));
+
+ out:
+  if (error != NULL) {
+    g_warning ("failed to save image: %s", error->message);
+    g_error_free (error);
+  }
+  g_clear_object (&io_stream);
+  g_clear_object (&file);
 }
 
 static void
@@ -309,6 +437,8 @@ local_create_user (GisAccountPageLocal *page)
   act_user_set_user_name (priv->act_user, username);
   act_user_set_account_type (priv->act_user, priv->account_type);
 
+  set_user_avatar (page);
+
   g_signal_emit (page, signals[USER_CREATED], 0, priv->act_user, "");
 }
 
@@ -319,12 +449,14 @@ gis_account_page_local_class_init (GisAccountPageLocalClass *klass)
 
   gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-account-page-local.ui");
 
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, avatar_button);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, avatar_image);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, subtitle);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, fullname_entry);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, username_combo);
 
   object_class->constructed = gis_account_page_local_constructed;
+  object_class->dispose = gis_account_page_local_dispose;
 
   signals[VALIDATION_CHANGED] = g_signal_new ("validation-changed", GIS_TYPE_ACCOUNT_PAGE_LOCAL,
                                               G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
